package mentalese

// RelationIndex must always point to a real relation!

type StackFrame struct {
	Relations      RelationSet
	RelationIndex  int
	InBindings     BindingSet
	InBindingIndex int
	HandlerIndex   int
	HandlerCount   int
	OutBindings    BindingSet
	Cursor         *StackFrameCursor
}

func NewStackFrame(relations RelationSet, bindings BindingSet) *StackFrame {
	return &StackFrame{
		Relations:      relations,
		InBindings:     bindings,
		OutBindings:    NewBindingSet(),
		HandlerCount:   0,
		InBindingIndex: 0,
		HandlerIndex:   0,
		RelationIndex:  0,
		Cursor:         NewStackFrameCursor(),
	}
}

func (f *StackFrame) UpdateMutableVariable(variable string, value Term) {
	for _, binding := range f.InBindings.GetAll() {
		if binding.ContainsVariable(variable) {
			binding.Set(variable, value)
		}
	}
	for _, binding := range f.OutBindings.GetAll() {
		if binding.ContainsVariable(variable) {
			binding.Set(variable, value)
		}
	}
}

func (f *StackFrame) IsDone() bool {
	return f.RelationIndex >= len(f.Relations)
}

func (f *StackFrame) GetCurrentRelation() Relation {
	return f.Relations[f.RelationIndex]
}

func (f *StackFrame) GetCurrentInBinding() Binding {
	return f.InBindings.Get(f.InBindingIndex)
}

func (f *StackFrame) AddOutBinding(outBinding Binding) {
	f.OutBindings.Add(outBinding)
}
